'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _acorn = require('acorn');

var _walk = require('acorn/dist/walk');

var _ramda = require('ramda');

var _ramda2 = _interopRequireDefault(_ramda);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// contract :: String -> Constructor -> a
var contract = _ramda2.default.curry(function (name, ctor, param) {
  return _ramda2.default.unless(_ramda2.default.is(ctor), function () {
    throw new TypeError('`' + name + '` should be `' + _ramda2.default.type(ctor()) + '`, but got `' + _ramda2.default.type(param) + '`');
  })(param);
});

var parse = function parse(input) {
  return (0, _acorn.parse)(input, { sourceType: 'module' });
};

// es2015+
var importValue = _ramda2.default.path(['source', 'value']);

// CommonJS
var isRequire = _ramda2.default.pathEq(['callee', 'name'], 'require');
var isLiteral = _ramda2.default.pipe(_ramda2.default.prop('arguments'), _ramda2.default.head, _ramda2.default.propEq('type', 'Literal'));
var isLiteralRequire = _ramda2.default.both(isRequire, isLiteral);
var requireValue = _ramda2.default.pipe(_ramda2.default.prop('arguments'), _ramda2.default.head, _ramda2.default.prop('value'));

// esDepsFromString -> String -> Array[String]
var esDepsFromString = _ramda2.default.unary(_ramda2.default.pipe(contract('input', String), parse, function (ast) {
  var list = [];
  var addToList = function addToList(item) {
    list = list.concat(item);
  };
  (0, _walk.simple)(ast, {
    ImportDeclaration: _ramda2.default.pipe(importValue, addToList),
    CallExpression: _ramda2.default.when(isLiteralRequire, _ramda2.default.pipe(requireValue, addToList))
  });
  return list;
}));

exports.default = esDepsFromString;
module.exports = exports['default'];
